<?php

/**
 * A mock implementation of MayI, which allows everything with no feedback.
 */
interface PermissionsInterface {

	/**
	 * Tell something about me (May I be, X?)
	 *
	 * Add some information about the environment. The actor is an object
	 * implementing the interface of op5MayI_Actor, and will be accessable
	 * through the name of the context.
	 *
	 * @param string $context
	 * @param op5MayI_Actor $actor
	 * @return op5mayi
	 */
	public function be($context, op5MayI_Actor $actor);

	/**
	 * Adds constaints to act upon, given an action.
	 * Constraints may be authorization, global configuration, or similar.
	 *
	 * Constraints are implemented through the interface of op5MayI_Constraints
	 *
	 * Priorities controls which constraints are allowed to export messages.
	 *
	 * Given that at least one constraint denies access, only messages from
	 * the denying constraints with highest available priority are returned.
	 * Several constraints can have equal, and thus highest, priority at the
	 * same time.
	 *
	 * Given that all constaints allows access, all messages from all
	 * constraints are returned.
	 *
	 * metrics is passed through in the same way as messages.
	 *
	 * @param op5MayI_Constraints $constraints
	 * @param integer $priority The priority of the constraint, default to 0
	 * @return op5mayi
	 */
	public function act_upon(op5MayI_Constraints $constraints, $priority = 0);


	/**
	 * Run an action, and return if you may do the given action.
	 *
	 * The method returns if an action is allowed to execute given the
	 * circumstanses of the enviornment (see be-method), and the constraints
	 * (see act_upon method)
	 *
	 * @param string $action
	 *        	the action in the format of "path.to.resource:action"
	 * @param array $override
	 *        values to override in the environement, which will be replaced
	 *        by the values in this array.
	 * @param array $messages
	 *        	returns a list of messages from constraints
	 * @param op5MayIMetric[] $metrics
	 *        	returns a list of performance data from constraints
	 * @return boolean
	 */
	public function run($action, array $override = array(), &$messages = false, &$metrics = false);

}
