<?php

require_once __DIR__.'/response.php';

/**
 * A special error response class
 *
 * Helps setting standardized API error keys
 */
class Api_Error_Response extends Api_Response
{
	/**
	 * Constructor
	 * @param $brief_msg String A brief message
	 * @param $code Integer The error code we should return
	 * @param $full_err String If false, we return the brief message. Otherwise,
	 *                  we give out the whole thing.
	 */
	function __construct($brief_msg, $code = 400, $full_err = false)
	{
		if ($full_err === false)
			$full_err = $brief_msg;
		parent::__construct(array('error' => $brief_msg, 'full_error' => $full_err), $code);
	}

	/**
	 * @param $output_format string
	 * @param $request Api_Request
	 * @return string
	 */
	protected function _get_adjusted_response($output_format, $request)
	{
		if ($output_format === 'html') {
			$this->title = $this->body['error'];
			if(isset($this->body['full_error'])) {
				$this->body = $this->body['full_error'];
			}
		}
		return parent::_get_adjusted_response($output_format, $request);
	}

	/**
	 * Include "usage" instructions together with your local error
	 *
	 * @param $brief_msg string
	 * @param $code int
	 * @param $request Api_Request
	 * @return Api_Error_Response
	 */
	public static function with_usage($brief_msg, $code, Api_Request $request)
	{
		$msg = ' ';
		if ($request->get_output_format() !== 'html')
			$msg = "Please visit {$request->get_base_url()}/api/help with a browser to see documentation";
		return new self($brief_msg, $code, $msg);
	}
}
