<?php

require_once 'op5/mayi.php';
require_once 'op5/config.php';

/**
 * Checks configuration for 'beta_features' and demands them to be activated
 * prior to allowing actions, such as viewing perfdata.
 */
class mayi_http_api_constraints implements op5mayi_constraints {
	function run($action, $env, &$messages, &$perfdata) {
		if(preg_match('/^monitor\.monitoring\.hosts\.(?P<section>\w+):/', $action, $matches)) {
			$config = op5config::instance()->getConfig('http_api.beta_features');
			$section = $matches['section'];
			if(isset($config[$section]) && !$config[$section]) {
				$messages[] = "Beta feature $section not enabled in /etc/op5/http_api.yml";
				op5log::instance('mayi')->log('notice', get_class($this)." denies '$action'. Add beta_features:\n  $section: true\nto /etc/op5/http_api.yml to allow these calls.");
				return false;
			}
		}
		if(preg_match('/^monitor\.license:(read|create)/', $action)) {
			$config = op5config::instance()->getConfig('http_api.beta_features');
			$section = 'license';
			if(isset($config[$section]) && !$config[$section]) {
				$messages[] = "Beta feature $section not enabled in /etc/op5/http_api.yml";
				op5log::instance('mayi')->log('notice', get_class($this)." denies '$action'. Add beta_features:\n  $section: true\nto /etc/op5/http_api.yml to allow these calls.");
				return false;
			}
		}
		return true;
	}
}

/**
 * Check either api-, app- or local usage, depending on the clients identification.
 *
 * @param $template_string string that contains %s to be replaced with 'api' and 'app'
 * @throws Api_Error_Response
 */
function mayi_run_api($template_string) {
	if(is_mobile()) {
		if(!op5mayi::instance()->run(sprintf($template_string, 'app'), array(), $messages)) {
			array_unshift($messages, "You are using a mobile application to access the Monitor API, which is not allowed. Please contact your op5 Monitor admin");
			throw new Api_Error_Response('Forbidden', 403, $messages);
		}
	} elseif(is_local()) {
		if(!op5mayi::instance()->run(sprintf($template_string, 'local'), array(), $messages)) {
			throw new Api_Error_Response('Forbidden', 403, $messages);
		}
	} else {
		if(!op5mayi::instance()->run(sprintf($template_string, 'api'), array(), $messages)) {
			throw new Api_Error_Response('Forbidden', 403, $messages);
		}
	}
}
