<h2>Get information about your license</h2>
<p>Perform a GET request to <em>/api/beta/license/data</em> to get a response
similar to this:</p>
<pre><samp>
{
	metadata: {
		branding: "trial",
		license_api_version: "trial"
	},
	expires_at: 1490565600
}
</samp></pre>
<p>The contents of <em>metadata</em> varies from license to license, you cannot
know on beforehand which of the fields that exists.</p>
<p>The contents of <em>expires_at</em> is a UNIX timestamp that you can use to
examine when the license expired/will expire.</p>

<h3>cURL in terminal</h3>
<div class='curl'>
<pre><code>curl --user 'your_username:your_password' --header 'accept: application/json' '<?php echo $license_endpoint_uri; ?>'</code></pre>
</div>

<h3>Python</h3>
<div class='python'>
	<pre><code>import requests
response = requests.get(
	'<?php echo $license_endpoint_uri; ?>',
	auth=('your_username', 'your_password'),
	headers={
		'accept': 'application/json'
	}
)

print response.text</code></pre>
</div>

<h3>PHP with cURL</h3>
<div class='php'>
	<pre><code>$a_handle = curl_init('<?php echo $license_endpoint_uri; ?>');
curl_setopt($a_handle, CURLOPT_USERPWD, 'your_username:your_password');
curl_setopt($a_handle, CURLOPT_RETURNTRANSFER, TRUE);
curl_setopt($a_handle, CURLOPT_HTTPHEADER, array('accept: application/json'));
$result = curl_exec($a_handle);
echo $result;</code></pre>
</div>

<h2>Upload a new License</h2>
<p>After you have retrieved a license file from OP5, you can perform a PUT
request to <em>/api/beta/license/data</em> with the contents of that file in
the request body.</p>
<p>If you upload a file that is not a valid license file, the HTTP response
will return an error code and a descriptive error message, such as this:
<pre><samp>{
	"error": "Bad Request",
	"full_error": "The uploaded file does not seem to be a valid license file. Please contact OP5 Support if you believe this is an error"
}</samp></pre>

<h3>cURL in terminal</h3>
<div class='curl'>
<pre><code>curl --user 'your_username:your_password' --header 'accept: application/json' --header 'content-type: application/octet-stream' --upload-file "your_license_file" '<?php echo $license_endpoint_uri; ?>'</code></pre>
</div>

<h3>Python</h3>
<div class='python'>
	<pre><code>import requests
response = requests.put(
	'<?php echo $license_endpoint_uri; ?>',
	auth=('your_username', 'your_password'),
	data=open('your_license_file', 'rb').read(),
	headers={
		'content-type': 'application/octet-stream',
		'accept': 'application/json'
	}
)

print response.text</code></pre>
</div>

<h3>PHP with cURL</h3>
<div class='php'>
	<pre><code>$license_file_handler = fopen('your_license_file', 'r');
$a_handle = curl_init('<?php echo $license_endpoint_uri; ?>');
curl_setopt($a_handle, CURLOPT_USERPWD, 'your_username:your_password');
curl_setopt($a_handle, CURLOPT_RETURNTRANSFER, TRUE);
curl_setopt($a_handle, CURLOPT_PUT, 1);
curl_setopt($a_handle, CURLOPT_INFILE, $license_file_handler);
curl_setopt($a_handle, CURLOPT_INFILESIZE, filesize('your_license_file'));
curl_setopt($a_handle, CURLOPT_HTTPHEADER, array(
	'Content-Type: application/octet-stream',
	'Accept: application/json'
));
$result = curl_exec($a_handle);
fclose($license_file_handler);
curl_close($a_handle);
echo $result;</code></pre>
</div>
