<?php

require_once __DIR__.'/lib/error_response.php';
require_once __DIR__.'/lib/proxy.php';
require_once __DIR__.'/lib/response.php';
require_once __DIR__.'/lib/request.php';

/**
 * @param $request HTTP request
 * @throws Api_Error_Response
 * @return Api_Response
 */
function generate_magellan_response($request) {
	$magellan = new Magellan();
	return $magellan->handle_request($request);
}

/**
 * Controller for the HTTP API to manage /api/magellan
 */
class Magellan extends ApiProxy {
	/**
	 * Main entry point for requests that should be handled by magellan
	 *
	 * @param Api_Request $request The HTTP request we're responding to
	 * @throws Api_Error_Response
	 * @returns Api_Response
	 */
	public function handle_request(Api_Request $request) {
		if ($request->is_help()) {
			return $this->help();
		}

		$endpoint = op5config::instance()->getConfig('http_api.magellan.endpoint');

		if ($endpoint == null) {
			$config_path = op5config::instance()->getPathForNamespace('http_api');
			throw new Api_Error_Response('Magellan endpoint not set', 500,
				"Magellan endpoint is not set. Make sure the endpoint is set in $config_path");
		}

		return $this->forward_request($request, $endpoint);
	}

	public function help() {
		// It isn't particulary nice to embed a standalone HTML document like this,
		// but browsers should handle it OK.
		$description = file_get_contents('/opt/magellan/api-definition.html');
		return new Api_Response(
			array(
				'description' => $description,
				'body' => '',
				'menu' => '',
				'properties' => ''
			)
		);
	}
}
