<?php

require_once __DIR__.'/lib/error_response.php';
require_once __DIR__.'/lib/proxy.php';
require_once __DIR__.'/lib/response.php';
require_once __DIR__.'/lib/request.php';

/**
 * @param $request HTTP request
 * @throws Api_Error_Response
 * @return Api_Response
 */
function generate_nachos_response($request) {
    $nachos = new Nachos();
    return $nachos->handle_request($request);
}

/**
 * Controller for the HTTP API to manage /api/nachos
 */
class Nachos extends ApiProxy {
    /**
     * Main entry point for requests that should be handled by nachos
     *
     * @param Api_Request $request The HTTP request we're responding to
     * @throws Api_Error_Response
     * @returns Api_Response
     */
    public function handle_request(Api_Request $request) {
        if ($request->is_help()) {
            return $this->help();
        }

        if ($request->get_method() == 'POST') {
            $body = $request->get_body();
            $username = $request->get_username();
            if (array_key_exists('user', $body) && $body['user'] != $username) {
                throw new Api_Error_Response('Unauthorized', 401,
                    "Provided user '${body['user']}' does not match authenticated user '$username'");
            }
        }

        $endpoint = op5config::instance()->getConfig('http_api.nachos.endpoint');

        if ($endpoint == null) {
            $config_path = op5config::instance()->getPathForNamespace('http_api');
            throw new Api_Error_Response('Nachos endpoint not set', 500,
                "Nachos endpoint is not set. Make sure the endpoint is set in $config_path");
        }

        return $this->forward_request($request, $endpoint);
    }

    public function help() {
        $description = "<a href='https://docs.itrsgroup.com/docs/op5-monitor/8.1.0/topics/api/monitor-api.html#Nachos_API'>See the Nachos KB article for more information.</a>";
        return new Api_Response(
            array(
                'description' => $description,
                'body' => '',
                'menu' => '',
                'properties' => ''
            )
        );
    }
}
