<?php

require_once 'op5/spyc.php';

function usage() {
	print <<<EOF
Usage: upgrade_config.php path/to/http_api.yml [output-file.yml]

	output-file.yml is optional and defaults to the http_api.yml
	(i.e. overwriting the input).

	The yml file(s) must be both readable and writable.

EOF;
}

/**
 * Do manual and hopefully smart updates of the config file
 */
function upgrade(array $config) {

	// Add auth_method
	if(!isset($config['auth_method'])) {
		$config['auth_method'] = array();
	}
	if(!isset($config['auth_method']['basic_auth'])) {
		$config['auth_method']['basic_auth'] = true;
	}
	if(!isset($config['auth_method']['ninja'])) {
		$config['auth_method']['ninja'] = true;
	}

	// Beta features
	if(!isset($config['beta_features'])) {
		$config['beta_features'] = array();
	}

	if(!isset($config['beta_features']['perfdata'])) {
		$config['beta_features']['perfdata'] = true;
	}
	if(!isset($config['beta_features']['license'])) {
		$config['beta_features']['license'] = true;
	}

	// Add forthcoming upgrades HERE

	return $config;
}

if(basename(__FILE__) != basename($argv[0])) {
	// we're being required or included, not called directly as a cli tool,
	// so let's not do anything fancy
	return;
}

if(PHP_SAPI != 'cli') {
	header("Location: /");
	exit(1);
}

if(!isset($argv[1]) || !is_readable($argv[1]) || !is_writable($argv[1])) {
	usage();
	exit(1);
}

$output_file = $argv[1];
if(isset($argv[2])) {
	if(!is_writable($argv[2])) {
		usage();
		exit(1);
	}
	$output_file = $argv[2];
}

$conf = spyc_load_file($argv[1]);
$conf = upgrade($conf);
$yaml = Spyc::YAMLDump($conf);
if(!file_put_contents($output_file, $yaml)) {
	echo "Could not save to file '$path'\n";
	exit(1);
}
