<?php

require_once __DIR__.'/../include/import.inc.php';

/**
 * HTTP API entry point
 */
class http_api_class
{
	private function index_html_response()
	{
		$help_class = new help_nacoma_Model();
		$body = "<h2>Using the <a href='http://en.wikipedia.org/wiki/Representational_state_transfer#RESTful_web_services' title='Representational state transfer'>REST</a> API</h2>

			<h3>Introduction</h3>

			<p>This is the REST-API that let's you configure op5 Monitor by issuing regular HTTP requests. Basically, you 'visit' an URI, which triggers Monitor to do something, and you get a response telling you what happened.</p>

			<p>You get help with the formulation of the requests on each object type's help page (look over to your left to find which object types are available). <strong>You also get tips on how to perform HTTP requests via the terminal (using cURL) and via scripting languages (using Python or PHP).</strong></p>

			<p>A final, important, note on using this configuration API: <strong>no changes are inserted until you manually confirm them</strong>, <a href='#persistent_changes'>instructions on saving changes are found below</a>.</p>

			<h3>Technical information</h3>

			<h4>Authentication</h4>

			<p>You send along a pair of regular op5 Monitor user credentials in a basic auth header with every request. View any example and you will understand how to do this.</p>

			<h4>Requests</h4>
			<p>In a REST-API, there are a couple of important things that make up for a valid request:</p>
			<ul>
				<li>
					The URI, in the following form
					<ul>
					<li>/api/config/host</li>
					<li>/api/config/host/a-hostname</li>
					</ul>
				</li>
				<li>
					The HTTP method
					<ul>
						<li>GET: lists objects.</li>
						<li>POST: creates objects.</li>
						<li>PUT: overwrite objects.</li>
						<li>PATCH: update objects.</li>
						<li>DELETE: delete objects.</li>
					</ul>
				</li>
				<li>The format as a HTTP header:
					<ul>
						<li><var>accept: application/xml<br />content-type: application/xml</var></li>
						<li><var>accept: application/json<br />content-type: application/xml</var></li>
						<li><var>accept: text/html<br />content-type: application/json</var> says: 'give me back HTML based on my input data in JSON'</li>
					</ul>
					or, appended to the URI:
					<ul>
						<li>?format=xml</li>
						<li>?format=json (default, except for requests from a browser)</li>
						<li>?format=html (default in a browser environment)</li>
					</ul>
				</li>
			</ul>

			<br />
			<h4>Important info about full (PUT) and partial (PATCH) updates on collection-type properties</h4>
			
			<p>
			These operation types not only overwrite current values but also causes detachment of template values, with one exception; passing an empty collection instead causes the opposite behavior, i.e. (re)attachment of template values.
			</p>
			
			<p>
			<u>Example</u>
			</p>
			
			<p>
			A Host is created with no <var>contacts</var> set, but makes use of a <var>template</var> with two <var>contacts</var>: user01, user02.
			<br />
			These <var>contacts</var> can be seen by querying for the Host:
			</p>
			
			<div class='curl'>
				<pre><code>curl https://monitor-server/api/config/host/srv01 -u '&lt;your username&gt;:&lt;your password&gt;'</code></pre>
			</div>
			<div>
			<pre><code>
{
	\"contacts\": [\"user01\", \"user02\"],
	...
}
			</code></pre>
			
			<p>
			In order to query for a document with values belonging directly to the Host, the <var>template_expand</var> parameter can be used:
			</p>
			<div class='curl'>
				<pre><code>curl https://monitor-server/api/config/host/srv01?template_expand=0 -u '&lt;your username&gt;:&lt;your password&gt;'</code></pre>
			</div>
			<pre><code>
{
   \"contacts\": [],
   ...
}
			</code></pre>

			<p>
			Performing a non-empty partial update on the Host's <var>contacts</var> field causes detachment of template values:
			</p>
			<div class='curl'>
				<pre><code>curl https://monitor-server/api/config/host/srv01 -X PATCH -d '{\"contacts\": [\"user03\"]}' -u '&lt;your username&gt;:&lt;your password&gt;'</code></pre>
			</div>
			<pre><code>
{
   \"contacts\": [\"user03\"],
   ...
}
			</code></pre>

			<p>
			...and an empty partial update causes the opposite behavior:
			</p>
			<div class='curl'>
				<pre><code>curl https://monitor-server/api/config/host/srv01 -X PATCH -d '{\"contacts\": []}' -u '&lt;your username&gt;:&lt;your password&gt;'</code></pre>
			</div>
			
			<pre><code>
{
   \"contacts\": [\"user01\", \"user02\"],
   ...
}
			</code></pre>
			</div>
						
			<h4>Responses</h4>
			<p>You need to examine two things in the response: the status code and the body. The status code tells you what Monitor did with your input and the body gives you a detailed response.</p>

			<p>Examples of status codes:</p>
			<ul>
				<li>200: Everything went fine.</li>
				<li>201: Everything went fine, you wanted to create an object and it was created.</li>
				<li>400: You sent a request Monitor didn't understand. Correct the input and try again.</li>
				<li>401: Authentication required.</li>
				<li>403: You're not authorized to do this.</li>
				<li>404: The URI doesn't exist.</li>
			</ul>

			<p>These status codes are general for all web traffic and can be found at the <a href='http://en.wikipedia.org/wiki/HTTP_status_code'>List of HTTP status codes</a></p>

			<h4 id='persistent_changes'>Making changes persistent</h4>
			<p>Just as in the web GUI of op5 Monitor, when you issue changes they are not immidiately inserted into the Nagios configuration files. An additional action is required and here is how you do it via the HTTP API:</p>
			<ul>
				<li>Perform a <strong>GET</strong> request to /api/config/change to see all the changes you have made, which are to be committed.
				<ul>
					<li>If you wish to throw away the changes: perform a <strong>DELETE</strong> request to /api/config/change, which undos all changes.</li>
					<li>If you are satisfied with the changes: perform a <strong>POST</strong> request to /api/config/change.</li>
				</ul>
				</li>
			</ul>";
		$response = array(
			'body' => $body,
			'menu' => $help_class->list_objects('/api/help/config/')
		);
		return $response;
	}

	/**
	 * Returns a properly formatted view of the object,
	 * expanded, without internal ID numbers, and with sub-objects
	 * included.
	 *
	 * @param $obj object_nacoma_Model
	 * @param $template_expand bool enable/disable template expansion
	 * @return array
	 */
	private function output_object(object_nacoma_Model $obj, $template_expand = true)
	{
		$obj->expand_object($template_expand);
		$res = $obj->denormalize();
		$prop = $obj->get_properties();
		if (isset($prop['deprecated_names'])) {
			foreach ($prop['deprecated_names'] as $old => $new) {
				if (isset($res[$new]))
					$res[$old] = $res[$new];
			}
		}
		unset($res['id']);
		foreach ($obj->find_slave_objects() as $type => $ary) {
			$res[$type.'s'] = array();
			foreach ($ary as $subobj) {
				$class = $type.'_nacoma_Model';
				$subobj = new $class($subobj);
				$subres = $this->output_object($subobj);
				unset($subres[$subobj->master_obj_identifier]);
				$res[$type.'s'][] = $subres;
			}
		}
		return $res;
	}

	/**
	 * Generate a map of objects from a posted info-map
	 *
	 * We have the following nesting to support today:
	 * host =>
	 *     hostdependency,
	 *     hostescalation,
	 *     service =>
	 *         servicedependency,
	 *         serviceescalation
	 */
	private function generate_objmap(object_nacoma_Model $obj, $info)
	{
		// Used for fake parent relations, this will be un-nested in save-objects
		// The idea is copied from host_wizard
		// Fun fact: $obj->obj['id'] may not be negative or validation fails.
		// $obj->id may, and everything breaks if it isn't set to something.
		static $next_id = -1;
		if ($obj->id === false || $obj->id === 'new')
			$obj->id = $next_id--;
		$all_referenced = array();
		if (is_array($obj->slave_types)) {
			foreach ($obj->slave_types as $type) {
				if (!isset($info[$type.'s']))
					continue;
				$all_referenced[$type] = array();
				if (!is_array($info[$type.'s'])) {
					throw new Api_Error_Response('Invalid request', 400, "Key '{$type}s' must contain a list");
				}
				foreach ($info[$type.'s'] as $subinfo) {
					if (!is_array($subinfo)) {
						throw new Api_Error_Response('Invalid request', 400, "Key '{$type}s' must contain a list");
					}
					$class = $type.'_nacoma_Model';
					$subobj = new $class();
					$subobj->set_master_object_type($obj->type);
					$subobj->set_master_object_id($obj->id);
					if ($type == 'service') {
						$subobj->get_object_by_name($info[$obj->properties['topic']].';'.$subinfo['service_description']);
					}
					else if ($type == 'hostdependency') {
						$hostgroup = 0;
						$host = 0;
						if (isset($subinfo['host_name']))
							$host = host_nacoma_Model::get_id_for($subinfo['host_name']);
						if (isset($subinfo['hostgroup_name']))
							$hostgroup = hostgroup_nacoma_Model::get_id_for($subinfo['hostgroup_name']);
						$query = "select id from $type where {$subobj->master_obj_identifier} = {$obj->id} and host_name = $host and hostgroup_name = $hostgroup";
						$res = nacoma_db::exec_query($query);
						$row = $res->fetch(PDO::FETCH_NUM);
						unset($res);
						if ($row) {
							$subobj->id = $row[0];
							$subobj->get_object();
						}
						unset($res);
					}
					else if ($type == 'servicedependency') {
						$service = service_nacoma_Model::get_id_for($subinfo['service']);
						$query = "select id from $type where {$subobj->master_obj_identifier} = {$obj->id} and service = $service";
						$res = nacoma_db::exec_query($query);
						$row = $res->fetch(PDO::FETCH_NUM);
						unset($res);
						if ($row) {
							$subobj->id = $row[0];
							$subobj->get_object();
						}
					}
					$sub_referenced = $this->generate_objmap($subobj, $subinfo);
					$all_referenced[$type][$sub_referenced['object']->id] = $sub_referenced;
				}
				unset($info[$type.'s']);
			}
		}
		$obj->normalize($info, true);
		$obj->get_default_object_sanely();
		$all_referenced['object'] = $obj;

		return $all_referenced;
	}

	/**
	 * We have arbitrarily decided that PUTing a sub-object element
	 * to an object means we need to delete any preexisting sub-objects
	 * of that type. That is, PUTing the host
	 * {'host_name':'foo','services':[],'hostdependencys':[{'host_name':'monitor'}]}
	 * means we delete any services attached to the object, we delete all
	 * dependencies and create a new one, but we do not touch hostescalations.
	 */
	private function purge_other_subs($objs)
	{
		$obj = $objs['object'];
		foreach ($obj->find_slave_objects() as $type => $ary) {
			$class = $type.'_nacoma_Model';
			if (isset($objs[$type])) {
				foreach ($ary as $obj_id) {
					if (isset($objs[$type][$obj_id])) {
						/* this is a service, clean service sub objs */
						$this->purge_other_subs($objs[$type][$obj_id]);
						continue;
					}
					$obj = new $class($obj_id);
					if (!$obj->banish_object()) {
						return array($obj->get_object_name() => $obj->delete_message);
					}
				}
			}
		}
		return true;
	}

	/**
	 * Baring bugs, this should detect any and all problems with all the listed
	 * objects, so we can bail out before we do any changes.
	 *
	 * @return array [object_name => [error1, errorN]]
	 */
	private function validate_objects($objs)
	{
		$obj = $objs['object'];
		unset($objs['object']);
		if (!$obj->validate_object()) {
			return array($obj->get_object_name() => $obj->validation_error_list);
		}
		if (empty($objs))
			return array();

		foreach ($objs as $type => $subobjs) {
			foreach ($subobjs as $subobj) {
				if ($res = $this->validate_objects($subobj))
					return $res;
			}
		}
		return array();
	}

	/**
	 * Before 6.3 we could immediately login after having created a user
	 * but since then, we added the 'modules' property which one needs in
	 * order to login at all (not just through the HTTP API).
	 *   - if you don't want the user to be able to login, send an empty
	 *   value for modules
	 *   - if you want the old behaviour (fallback to default login
	 *   method), don't pass the modules parameter, and we'll try to find
	 *   a viable module for that user
	 *
	 * @param $type string user, host, service etc
	 * @param $body array the object's properties (POST params)
	 * @return array of object properties
	 */
	private function shim_user_no_modules($type, $body) {
		if($type != 'user' || isset($body['modules'])) {
			return $body;
		}
		$modules_to_add = op5auth::instance()->get_metadata('require_user_configuration');
		if(!$modules_to_add) {
			return $body;
		}
		$body['modules'] = $modules_to_add;
		return $body;
	}

	/**
	 * Validate that all objects in $objs can be saved, and then save them.
	 */
	private function save_objects($objs)
	{
		$obj = $objs['object'];
		unset($objs['object']);
		if ($obj->id < 0)
			$obj->id = false;
		if (!$obj->save_object()) {
			/* this shouldn't happen, but who knows? */
			return array($obj->get_object_name() => $obj->validation_error_list);
		}
		if (empty($objs))
			return true;

		foreach ($objs as $type => $subobjs) {
			foreach ($subobjs as $subobj) {
				$subobj['object']->set_master_object_id($obj->id);
				if (($res = $this->save_objects($subobj)) !== true)
					return $res;
			}
		}
		return true;
	}

	/**
	 * This will be executed for calls to the /change/ pseudo-type
	 *
	 * This allows users to GET pending changes, POST them to disk, and DELETE
	 * them all to restore a previous state.
	 */
	protected function handle_change(Api_Request $request)
	{
		// TODO Since there's no exact 1:1 mapping of the export action
		// in Nacoma (independent of objects to export), we keep this
		// around even after mayi is introduced
		if (!in_array(NACOMA_ACCESS_FULL_ACCESS, get_nacoma_access('export')))
			return new Api_Error_Response("You're not authorized to save config changes", 403);

		switch($_SERVER['REQUEST_METHOD']) {
		 case 'GET':
			$cl = new Changelog_nacoma_Model();
			return new Api_Response($cl->get_list());
		 case 'POST':
			// This is a possible API break. Before 2015.a, you
			// could export without having hosts:create rights
			// (such as only api_config and export rights), but
			// now we have to check *something* in order to have
			// the license check kick in and restrict on the
			// number of hosts in the config-after-save config.
			$env = array('monitor' => array('monitoring' => array('hosts' => get_num_hosts_configured(), 'services' => get_num_services_configured())));
			foreach(array('hosts', 'services') as $objtype) {
				if (!op5mayi::instance()->run("monitor.monitoring.$objtype:create.api.configuration", $env, $message)) {
					return new Api_Error_Response('Forbidden', 403, $message);
				}
			}
			$cl = new Changelog_nacoma_Model();
			if (($err = do_export($cl)) === true) {
				switch(export_reload_monitor()) {
				 case 1:
					return new Api_Error_Response("Monitor doesn't appear to be running", 500, "This could indicate a more serious error, so restarting not attempted.");
				 case 2:
					return new Api_Error_Response("Failed to open monitor command pipe for writing", 500, "This can happen when Monitor isn't running.");
				 default:
					return new Api_Response("Changes saved");
				}
			} else {
				return new Api_Error_Response('Export failed', 500, $err);
			}
		 case 'DELETE':
			if(!revert_users_changes()) {
				return new Api_Error_Response("The database is currently being imported or exported", 500, "This should merely take a few seconds, but while it's going on, you cannot delete the pending configuration");
			}
			return new Api_Response("Changes reverted");
		 default:
			return new Api_Error_Response('Invalid method "'.$_SERVER['REQUEST_METHOD'].'"', 405, "Only GET, POST and DELETE requests are allowed when handling changes");
		}
	}

	/**
	 * This will be executed for help calls
	 */
	protected function handle_help(Api_Request $request, $type)
	{
		if ($_SERVER['REQUEST_METHOD'] !== 'GET') {
			return Api_Error_Response::with_usage('Invalid request', 400, $request);
		}
		$help_class = new help_nacoma_Model();
		if (!$type) {
			return new Api_Response($this->index_html_response(), 200);
		} else if (array_key_exists($type, $help_class->get_available_objects())) {
			return new Api_Response($help_class->get_html_help_for_object_type($type), 200);
		} else {
			return Api_Error_Response::with_usage("Invalid object type given: '$type'", 404, $request);
		}
	}

	/**
	 * This will be executed for "regular" API requests that show/create/modify
	 * nacoma objects
	 */
	protected function handle_regular(Api_Request $request, $type, $name=false)
	{
		if (($owner = nacoma_objlock::get_lock_owner('all', 0))) {
			return new Api_Error_Response("The database is currently being imported or exported by $owner", 503, "This should merely take a few seconds, but while it's going on, no other users may edit the database.");
		}

		$class = $type . '_nacoma_Model';
		$obj = new $class();
		$params = $request->get_params();

		if (isset($params['parent_type'])) {
			// this is used for accessing for example hostgroups' services,
			// since /api/config/service/a_host;a_service is default (in contrast to
			// /api/config/service/a_hostgroup;another_service?parent_type=hostgroup)
			$obj->set_master_object_type($params['parent_type']);
		}
		if ($name)
			$obj->get_object_by_name($name);

		$object_types_without_resource = array(
			'servicedependency',
			'hostdependency',
			'serviceescalation',
			'hostescalation'
		);

		$mayi_resource = $obj->mayi_resource();
		if ($name === false) {
			switch ($_SERVER['REQUEST_METHOD']) {
			 case 'GET':
				mayi_run_api($mayi_resource.":read.%s.configuration");
				$objs = $obj::get_objects_for_user();
				$out = array();
				if (is_array($objs)) {
					foreach ($objs as $obj) {
						if(in_array($type, $object_types_without_resource)) {
							$out[] = array('name' => $obj);
						} else {
							$out[] = array('name' => $obj, 'resource' => rtrim($request->get_full_url(), '/').'/'.urlencode(urlencode($obj)));
						}
					}
				}
				return new Api_Response($out);
			 case 'POST':
				mayi_run_api($mayi_resource.":create.%s.configuration");
				if (!$obj->can_user_write('new'))
					return new Api_Error_Response('Permission Denied', 403);
				try {
					$body = $request->get_body();
				} catch(Exception $e) {
					return new Api_Error_Response('Bad Request', 400, $e->getMessage());
				}
				try {
					$body = $this->shim_user_no_modules($type, $body);
					$objs = $this->generate_objmap($obj, $body);
					$existing = new $class();
					unset($objs['object']->topic);
					$existing->get_object_by_name($objs['object']->get_object_name());
					unset($objs['object']->obj['id']);
					if ($existing->id !== false && $existing->id !== 'new') {
					    # Should not return 'Object already exists' if it's a hostgroup service,
                        if ($existing instanceof service_nacoma_Model) {
                            if ($existing::get_parent_type($existing->id) != 'hostgroup') {
                                return new Api_Error_Response('Object already exists', 409);
                            }
                        } else {
                            return new Api_Error_Response('Object already exists', 409);
                        }
                    }
					if ($errs = $this->validate_objects($objs))
						return new Api_Error_Response("Couldn't validate object", 400, $errs);
					if (isset($params['parent_type']) && $objs['object']->master_obj_type !== $params['parent_type'])
						return new Api_Error_Response("Bad arguments", 400, "You asked for an object with {$params['parent_type']} parent, but found an object with {$objs['object']->master_obj_type} parent");
					if (($errs = $this->save_objects($objs)) !== true)
						return new Api_Error_Response("Couldn't save object", 400, $errs);
					return new Api_Response($this->output_object($obj), 201);
				} catch (NacomaRelationException $ex) {
					return new Api_Error_Response("Couldn't save object", 400, $ex->getMessage());
				}
			 default:
				return new Api_Error_Response('Invalid method "'.$_SERVER['REQUEST_METHOD'].'"', 405, "Only GET and POST requests are allowed when querying a type of resource ($type) instead of a named resource ($type/an_object)");
			}
		}

		if (isset($params['parent_type']) && $obj->master_obj_type !== $params['parent_type'])
			return new Api_Error_Response("Bad arguments", 400, "You asked for an object with {$params['parent_type']} parent, but found an object with {$obj->master_obj_type} parent");

		switch ($_SERVER['REQUEST_METHOD']) {
		 case 'GET':
			mayi_run_api($mayi_resource.":read.%s.configuration");
			if (!$obj->can_user_read($obj->id))
				return new Api_Error_Response('Permission Denied', 403);
			if (!$obj->id || $obj->id === 'new')
				return new Api_Error_Response('Object not found', 404);

			if(isset($params['template_expand'])) {
				/**
				 * Validate the `template_expand` parameter as a boolean option.
				 *
				 * TRUE for "1", "true", "on" and "yes"
				 * FALSE for "0", "false", "off" and "no"
				 * NULL otherwise
				 */
				$template_expand = filter_var($params['template_expand'], FILTER_VALIDATE_BOOLEAN);
			} else {
				$template_expand = true;
			}

			return new Api_Response($this->output_object($obj, $template_expand));
		 case 'PATCH':
			mayi_run_api($mayi_resource.":update.%s.configuration");
			if (!$obj->id || $obj->id === 'new')
				return new Api_Error_Response('Object not found', 404);
			if (!$obj->can_user_write($obj->id))
				return new Api_Error_Response('Permission Denied', 403);
			try {
				$body = $request->get_body();
			} catch(Exception $e) {
				return new Api_Error_Response('Bad Request', 400, $e->getMessage());
			}
			try {
				$objs = $this->generate_objmap($obj, $body);
				if (count($objs) > 1)
					return new Api_Error_Response("You may not PATCH sub-objects", 501);
				if ($errs = $this->validate_objects($objs))
					return new Api_Error_Response("Couldn't save object", 400, $errs);
				if (($errs = $this->save_objects($objs)) !== true)
					return new Api_Error_Response("Couldn't save object", 400, $errs);
				return new Api_Response($this->output_object($obj), 200);
			} catch (NacomaRelationException $ex) {
				return new Api_Error_Response("Couldn't save object", 400, $ex->getMessage());
			}
		 case 'PUT':
			mayi_run_api($mayi_resource.":update.%s.configuration");
			if (!$obj->can_user_write($obj->id))
				return new Api_Error_Response('Permission Denied', 403);
			try {
				$body = $request->get_body();
			} catch(Exception $e) {
				return new Api_Error_Response('Bad Request', 400, $e->getMessage());
			}
			try {
				$obj->get_default_object();
				$objs = $this->generate_objmap($obj, $body);
				if ($errs = $this->validate_objects($objs))
					return new Api_Error_Response("Couldn't save object", 400, $errs);
				if (($errs = $this->purge_other_subs($objs)) !== true)
					return new Api_Error_Response("Couldn't save object", 400, $errs);
				if (($errs = $this->save_objects($objs)) !== true)
					return new Api_Error_Response("Couldn't save object", 400, $errs);
				return new Api_Response($this->output_object($obj), 201);
			} catch (NacomaRelationException $ex) {
				return new Api_Error_Response("Couldn't save object", 400, $ex->getMessage());
			}
		 case 'DELETE':
			mayi_run_api($mayi_resource.":delete.%s.configuration");
			if (!$obj->can_user_write($obj->id))
				return new Api_Error_Response('Permission Denied', 403);
			if (!$obj->id || $obj->id === 'new')
				return new Api_Error_Response('Object not found', 404);
			if ($obj->banish_object())
				 return new Api_Response("$type $name deleted");
			else
				 return new Api_Error_Response("Couldn't delete $type $name", 400, $obj->delete_message);
		 default:
			return new Api_Error_Response('Invalid method "'.$_SERVER['REQUEST_METHOD'].'"', 405, "Only GET, PATCH, PUT and DELETE requests are allowed when querying a named resource ($type/$name) instead of a resource type ($type)");
		}
	}

	/**
	 * Main worker for this class, takes a request and returns a response
	 */
	public function handle_request(Api_Request $request)
	{
		if (!get_last_import_time() || changes_since_last_import()) {
			if(!import_config($request->get_username(), false, $error_message)) {
				return new Api_Error_Response("Automatic configuration import failed", 500, $error_message);
			}
		}

		$path = $request->get_path();
		$type = false;
		if (!empty($path))
			$type = array_shift($path);

		if ($request->is_help()) {
			return $this->handle_help($request, htmlspecialchars($type));
		}
		else if (!$type) {
			return Api_Error_Response::with_usage('Missing object type', 400, $request);
		}

		$class = $type . '_nacoma_Model';
		$name = false;
		if (!empty($path))
			$name = array_shift($path);

		if (!empty($path))
			return Api_Error_Response::with_usage('Invalid request', 404, $request);

		if ($type === 'change') {
			return $this->handle_change($request, $type);
		} else if (array_key_exists($type, Module_Manifest_Model::get('http_api_exposed_classes'))) {
			return $this->handle_regular($request, $type, $name);
		} else if ($type == 'role') {
			return new Api_Error_Response("Role objects no longer exists", 404);
		} else {
			return Api_Error_Response::with_usage('Invalid object type "'.$type.'"', 404, $request);
		}
	}
}
